import { LayoutViewer } from '../index';
import { L10n } from '@syncfusion/ej2-base';
import { ChangeEventArgs } from '@syncfusion/ej2-buttons';
import { Tab, SelectEventArgs } from '@syncfusion/ej2-navigations';
/**
 * Options Pane class.
 */
export declare class OptionsPane {
    private viewer;
    /**
     * @private
     */
    optionsPane: HTMLElement;
    /**
     * @private
     */
    isOptionsPaneShow: boolean;
    private resultsListBlock;
    private messageDiv;
    private results;
    private searchInput;
    private searchDiv;
    private searchTextBoxContainer;
    private replaceWith;
    private findDiv;
    private replaceDiv;
    private replaceButton;
    private replaceAllButton;
    private occurrenceDiv;
    private findOption;
    private matchCase;
    private wholeWord;
    private searchText;
    private resultsText;
    private messageDivText;
    private replaceButtonText;
    private replaceAllButtonText;
    private focusedIndex;
    private focusedElement;
    private resultContainer;
    private navigateToPreviousResult;
    private navigateToNextResult;
    private closeButton;
    private isOptionsPane;
    private findTab;
    private findTabButton;
    private replaceTabButton;
    private searchIcon;
    private matchDiv;
    private replacePaneText;
    private findPaneText;
    private matchDivReplaceText;
    private matchInput;
    private wholeInput;
    private regularInput;
    /**
     * @private
     */
    tabInstance: Tab;
    private findTabContentDiv;
    private replaceTabContentDiv;
    private findTabButtonHeader;
    private replaceTabButtonHeader;
    /**
     * @private
     */
    isReplace: boolean;
    private localeValue;
    /**
     * Constructor for Options pane module
     * @param {LayoutViewer} layoutViewer
     * @private
     */
    constructor(layoutViewer: LayoutViewer);
    /**
     * Get the module name.
     */
    private getModuleName;
    /**
     * Initialize the options pane.
     * @param {L10n} localeValue - Specifies the localization based on culture.
     * @private
     */
    initOptionsPane(localeValue: L10n, isRtl?: boolean): void;
    /**
     * Create replace pane instances.
     */
    private createReplacePane;
    /**
     * Gets selected tab item which tab is selected.
     * @param {SelectEventArgs} args - Specifies which tab will be opened.
     * @private
     */
    selectedTabItem: (args: SelectEventArgs) => void;
    private searchOptionChange;
    private navigateSearchResult;
    /**
     * Apply find option based on whole words value.
     * @param {ChangeEventArgs} args - Specifies the search options value.
     * @private
     */
    wholeWordsChange: (args: ChangeEventArgs) => void;
    /**
     * Apply find option based on match value.
     * @param {ChangeEventArgs} args - Specifies the search options value.
     * @private
     */
    matchChange: (args: ChangeEventArgs) => void;
    /**
     * Apply find options based on regular value.
     * @param {ChangeEventArgs} args - Specifies the search options value.
     * @private
     */
    /**
     * Binding events from the element when optins pane creation.
     * @private
     */
    onWireEvents: () => void;
    /**
     * Fires on key down actions done.
     * @private
     */
    onKeyDownInternal(): void;
    /**
     * Enable find pane only.
     * @private
     */
    onFindPane: () => void;
    private getMessageDivHeight;
    private onEnableDisableReplaceButton;
    /**
     * Enable replace pane only.
     * @private
     */
    onReplacePane: () => void;
    /**
     * Fires on key down on options pane.
     * @param {KeyboardEvent} event - Specifies the focus of current element.
     * @private
     */
    onKeyDownOnOptionPane: (event: KeyboardEvent) => void;
    /**
     * Fires on replace.
     * @private
     */
    onReplaceButtonClick: () => void;
    /**
     * Fires on replace all.
     * @private
     */
    onReplaceAllButtonClick: () => void;
    /**
     * Replace all.
     * @private
     */
    replaceAll(): void;
    /**
     * Fires on search icon.
     * @private
     */
    searchIconClickInternal: () => void;
    /**
     * Fires on getting next results.
     * @private
     */
    navigateNextResultButtonClick: () => void;
    private updateListItems;
    /**
     * Fires on getting previous results.
     * @private
     */
    navigatePreviousResultButtonClick: () => void;
    /**
     * Scrolls to position.
     * @param {HTMLElement} list - Specifies the list element.
     * @private
     */
    scrollToPosition(list: HTMLElement): void;
    /**
     * Fires on key down
     * @param {KeyboardEvent} event - Speficies key down actions.
     * @private
     */
    onKeyDown: (event: KeyboardEvent) => void;
    /**
     * Clear the focus elements.
     * @private
     */
    clearFocusElement(): void;
    /**
     * Close the optios pane.
     * @private
     */
    close: () => void;
    /**
     * Fires on results list block.
     * @param {MouseEvent} args - Specifies which list was clicked.
     * @private
     */
    resultListBlockClick: (args: MouseEvent) => void;
    /**
     * Show or hide option pane based on boolean value.
     * @param {boolean} show - Specifies showing or hiding the options pane.
     * @private
     */
    showHideOptionsPane(show: boolean): void;
    /**
     * Clears search results.
     * @private
     */
    clearSearchResultItems(): void;
    /**
     * Dispose the internal objects which are maintained.
     * @private
     */
    destroy(): void;
    /**
     * Dispose the internal objects which are maintained.
     */
    private destroyInternal;
}
