import { Query, DataManager } from '@syncfusion/ej2-data';
import { IDataProcessor, IGrid, PendingState } from '../base/interface';
import { PredicateModel } from '../base/grid-model';
import { ServiceLocator } from '../services/service-locator';
/**
 * Grid data module is used to generate query and data source.
 * @hidden
 */
export declare class Data implements IDataProcessor {
    dataManager: DataManager;
    protected parent: IGrid;
    protected serviceLocator: ServiceLocator;
    protected dataState: PendingState;
    /**
     * Constructor for data module.
     * @hidden
     */
    constructor(parent?: IGrid, serviceLocator?: ServiceLocator);
    private reorderRows;
    protected getModuleName(): string;
    /**
     * The function used to initialize dataManager and external query
     * @return {void}
     */
    private initDataManager;
    /**
     * The function is used to generate updated Query from Grid model.
     * @return {Query}
     * @hidden
     */
    generateQuery(skipPage?: boolean): Query;
    protected aggregateQuery(query: Query, isForeign?: boolean): Query;
    protected virtualGroupPageQuery(query: Query): Query;
    protected pageQuery(query: Query, skipPage?: boolean): Query;
    protected groupQuery(query: Query): Query;
    protected sortQuery(query: Query): Query;
    protected searchQuery(query: Query): Query;
    protected filterQuery(query: Query, column?: PredicateModel[], skipFoerign?: boolean): Query;
    private fGeneratePredicate;
    /**
     * The function is used to get dataManager promise by executing given Query.
     * @param  {Query} query - Defines the query which will execute along with data processing.
     * @return {Promise<Object>}
     * @hidden
     */
    getData(args?: {
        requestType?: string;
        foreignKeyData?: string[];
        data?: Object;
        index?: number;
    }, query?: Query): Promise<Object>;
    private insert;
    private executeQuery;
    private formatGroupColumn;
    private crudActions;
    /** @hidden */
    saveChanges(changes: Object, key: string, original: Object, query?: Query): Promise<Object>;
    private getKey;
    /** @hidden */
    isRemote(): boolean;
    private addRows;
    private removeRows;
    private getColumnByField;
    protected destroy(): void;
    getState(): PendingState;
    setState(state: PendingState): Object;
    getStateEventArgument(query: Query): PendingState;
    private eventPromise;
    /**
     * Gets the columns where searching needs to be performed from the Grid.
     * @return {string[]}
     */
    private getSearchColumnFieldNames;
    private refreshFilteredCols;
}
