import { IPosition, IGrid, IRow, IExpandedRow } from './interface';
import { ServiceLocator } from '../services/service-locator';
import { Predicate } from '@syncfusion/ej2-data';
import { Column } from '../models/column';
import { Row } from '../models/row';
import { ColumnModel, AggregateColumnModel } from '../models/models';
import { AggregateType, HierarchyGridPrintMode } from './enum';
import { Dialog, Popup } from '@syncfusion/ej2-popups';
import { PredicateModel } from './grid-model';
/**
 * Function to check whether target object implement specific interface
 * @param  {Object} target
 * @param  {string} checkFor
 * @returns no
 * @hidden
 */
export declare function doesImplementInterface(target: Object, checkFor: string): boolean;
/**
 * Function to get value from provided data
 * @param  {string} field
 * @param  {Object} data
 * @param  {IColumn} column
 * @hidden
 */
export declare function valueAccessor(field: string, data: Object, column: ColumnModel): Object;
/**
 * The function used to update Dom using requestAnimationFrame.
 * @param  {Function} fn - Function that contains the actual action
 * @return {Promise<T>}
 * @hidden
 */
export declare function getUpdateUsingRaf<T>(updateFunction: Function, callBack: Function): void;
/**
 * @hidden
 */
export declare function updatecloneRow(grid: IGrid): void;
export declare function getCollapsedRowsCount(val: Row<Column>, grid: IGrid): number;
/**
 * @hidden
 */
export declare function recursive(row: Object[]): void;
/**
 * @hidden
 */
export declare function iterateArrayOrObject<T, U>(collection: U[], predicate: (item: Object, index: number) => T): T[];
/** @hidden */
export declare function iterateExtend(array: Object[]): Object[];
/** @hidden */
export declare function templateCompiler(template: string): Function;
/** @hidden */
export declare function setStyleAndAttributes(node: Element, customAttributes: {
    [x: string]: Object;
}): void;
/** @hidden */
export declare function extend(copied: Object, first: Object, second?: Object, exclude?: string[]): Object;
/** @hidden */
export declare function setColumnIndex(columnModel: Column[], ind?: number): number;
/** @hidden */
export declare function prepareColumns(columns: Column[] | string[] | ColumnModel[], autoWidth?: boolean): Column[];
/** @hidden */
export declare function setCssInGridPopUp(popUp: HTMLElement, e: MouseEvent | TouchEvent, className: string): void;
/** @hidden */
export declare function getActualProperties<T>(obj: T): T;
/** @hidden */
export declare function parentsUntil(elem: Element, selector: string, isID?: boolean): Element;
/** @hidden */
export declare function getElementIndex(element: Element, elements: Element[]): number;
/** @hidden */
export declare function inArray(value: Object, collection: Object[]): number;
/** @hidden */
export declare function getActualPropFromColl(collection: Object[]): Object[];
/** @hidden */
export declare function removeElement(target: Element, selector: string): void;
/** @hidden */
export declare function getPosition(e: MouseEvent | TouchEvent): IPosition;
/** @hidden */
export declare function getUid(prefix: string): string;
/** @hidden */
export declare function appendChildren(elem: Element | DocumentFragment, children: Element[] | NodeList): Element;
/** @hidden */
export declare function parents(elem: Element, selector: string, isID?: boolean): Element[];
/** @hidden */
export declare function calculateAggregate(type: AggregateType | string, data: Object, column?: AggregateColumnModel, context?: Object): Object;
/** @hidden */
export declare function getScrollBarWidth(): number;
/** @hidden */
export declare function getRowHeight(element?: HTMLElement): number;
/** @hidden */
export declare function isComplexField(field: string): boolean;
/** @hidden */
export declare function getComplexFieldID(field?: string): string;
/** @hidden */
export declare function setComplexFieldID(field?: string): string;
/** @hidden */
export declare function isEditable(col: Column, type: string, elem: Element): boolean;
/** @hidden */
export declare function isActionPrevent(inst: IGrid): boolean;
/** @hidden */
export declare function wrap(elem: any, action: boolean): void;
/** @hidden */
export declare function setFormatter(serviceLocator?: ServiceLocator, column?: Column): void;
/** @hidden */
export declare function addRemoveActiveClasses(cells: Element[], add: boolean, ...args: string[]): void;
/** @hidden */
export declare function distinctStringValues(result: string[]): string[];
/** @hidden */
export declare function getFilterMenuPostion(target: Element, dialogObj: Dialog, grid: IGrid): void;
/** @hidden */
export declare function getZIndexCalcualtion(args: {
    popup: Popup;
}, dialogObj: Dialog): void;
/** @hidden */
export declare function toogleCheckbox(elem: Element): void;
/** @hidden */
export declare function createCboxWithWrap(uid: string, elem: Element, className?: string): Element;
/** @hidden */
export declare function removeAddCboxClasses(elem: Element, checked: boolean): void;
/**
 * Refresh the Row model's foreign data.
 * @param row - Grid Row model object.
 * @param columns - Foreign columns array.
 * @param data - Updated Row data.
 * @hidden
 */
export declare function refreshForeignData(row: IRow<Column>, columns: Column[], data: Object): void;
/**
 * Get the foreign data for the corresponding cell value.
 * @param column - Foreign Key column
 * @param data - Row data.
 * @param lValue - cell value.
 * @param foreignData - foreign data source.
 * @hidden
 */
export declare function getForeignData(column: Column, data?: Object, lValue?: string | number, foreignKeyData?: Object[]): Object[];
/**
 * To use to get the column's object by the foreign key value.
 * @param foreignKeyValue - Defines ForeignKeyValue.
 * @param columns - Array of column object.
 * @hidden
 */
export declare function getColumnByForeignKeyValue(foreignKeyValue: string, columns: Column[]): Column;
/**
 * @hidden
 * @param filterObject - Defines predicate model object
 */
export declare function getDatePredicate(filterObject: PredicateModel, type?: string): Predicate;
/**
 * @hidden
 */
export declare function renderMovable(ele: Element, frzCols: number): Element;
/**
 * @hidden
 */
export declare function isGroupAdaptive(grid: IGrid): boolean;
/**
 * @hidden
 */
export declare function getObject(field?: string, object?: Object): any;
/**
 * @hidden
 */
export declare function getCustomDateFormat(format: string | Object, colType: string): string;
/**
 * @hidden
 */
export declare function getExpandedState(gObj: IGrid, hierarchyPrintMode: HierarchyGridPrintMode): {
    [index: number]: IExpandedRow;
};
/**
 * @hidden
 */
export declare function getPrintGridModel(gObj: IGrid, hierarchyPrintMode?: HierarchyGridPrintMode): IGrid;
/**
 * @hidden
 */
export declare function extendObjWithFn(copied: Object, first: Object, second?: Object, deep?: boolean): Object;
/**
 * @hidden
 */
export declare function measureColumnDepth(column: Column[]): number;
/**
 * @hidden
 */
export declare function checkDepth(col: Column, index: number): number;
/**
 * @hidden
 */
export declare function refreshFilteredColsUid(gObj: IGrid, filteredCols: PredicateModel[]): void;
/** @hidden */
export declare namespace Global {
    let timer: Object;
}
