import { IRenderer, IGrid, NotifyArgs, IModelGenerator } from '../base/interface';
import { Column } from '../models/column';
import { Row } from '../models/row';
import { ServiceLocator } from '../services/service-locator';
/**
 * Content module is used to render grid content
 * @hidden
 */
export declare class ContentRender implements IRenderer {
    private contentTable;
    private contentPanel;
    private rows;
    private freezeRows;
    private movableRows;
    private rowElements;
    private freezeRowElements;
    private index;
    colgroup: Element;
    private isLoaded;
    private tbody;
    private drop;
    private args;
    private rafCallback;
    protected parent: IGrid;
    private serviceLocator;
    private ariaService;
    protected generator: IModelGenerator<Column>;
    /**
     * Constructor for content renderer module
     */
    constructor(parent?: IGrid, serviceLocator?: ServiceLocator);
    /**
     * The function is used to render grid content div
     */
    renderPanel(): void;
    /**
     * The function is used to render grid content table
     */
    renderTable(): void;
    /**
     * The function is used to create content table elements
     * @return {Element}
     * @hidden
     */
    createContentTable(id: String): Element;
    private splitRows;
    /**
     * Refresh the content of the Grid.
     * @return {void}
     */
    refreshContentRows(args?: NotifyArgs): void;
    appendContent(tbody: Element, frag: DocumentFragment, args: NotifyArgs): void;
    /**
     * Get the content div element of grid
     * @return {Element}
     */
    getPanel(): Element;
    /**
     * Set the content div element of grid
     * @param  {Element} panel
     */
    setPanel(panel: Element): void;
    /**
     * Get the content table element of grid
     * @return {Element}
     */
    getTable(): Element;
    /**
     * Set the content table element of grid
     * @param  {Element} table
     */
    setTable(table: Element): void;
    /**
     * Get the Row collection in the Grid.
     * @returns {Row[] | HTMLCollectionOf<HTMLTableRowElement>}
     */
    getRows(): Row<Column>[] | HTMLCollectionOf<HTMLTableRowElement>;
    /**
     * Get the Movable Row collection in the Freeze pane Grid.
     * @returns {Row[] | HTMLCollectionOf<HTMLTableRowElement>}
     */
    getMovableRows(): Row<Column>[] | HTMLCollectionOf<HTMLTableRowElement>;
    /**
     * Get the content table data row elements
     * @return {Element}
     */
    getRowElements(): Element[];
    /**
     * Get the Freeze pane movable content table data row elements
     * @return {Element}
     */
    getMovableRowElements(): Element[];
    /**
     * Get the content table data row elements
     * @return {Element}
     */
    setRowElements(elements: Element[]): void;
    /**
     * Get the header colgroup element
     * @returns {Element}
     */
    getColGroup(): Element;
    /**
     * Set the header colgroup element
     * @param {Element} colgroup
     * @returns {Element}
     */
    setColGroup(colGroup: Element): Element;
    /**
     * Function to hide content table column based on visible property
     * @param  {Column[]} columns?
     */
    setVisible(columns?: Column[]): void;
    /**
     * @hidden
     */
    setDisplayNone(tr: Object, idx: number, displayVal: string, rows: Row<Column>[]): void;
    private colGroupRefresh;
    private initializeContentDrop;
    private canSkip;
    getModelGenerator(): IModelGenerator<Column>;
    renderEmpty(tbody: HTMLElement): void;
    setSelection(uid: string, set: boolean, clearAll?: boolean): void;
    getRowByIndex(index: number): Element;
    getVirtualRowIndex(index: number): number;
    getMovableRowByIndex(index: number): Element;
    private enableAfterRender;
    setRowObjects(rows: Row<Column>[]): void;
}
