import { Component, Property, ChildProperty, INotifyPropertyChanged, NotifyPropertyChanges, Animation } from '@syncfusion/ej2-base';import { Browser, isNullOrUndefined as isNOU,  getUniqueID, formatUnit, EventHandler } from '@syncfusion/ej2-base';import { EmitType, Collection, Complex, setStyleAttribute, Event, Effect, detach, AnimationModel } from '@syncfusion/ej2-base';import { attributes, extend, closest, compile as templateCompiler, classList, BaseEventArgs, isUndefined} from '@syncfusion/ej2-base';import { SwipeEventArgs, Touch, updateBlazorTemplate, isBlazor } from '@syncfusion/ej2-base';import { ButtonModel, Button  } from '@syncfusion/ej2-buttons';import { getZindexPartial } from '@syncfusion/ej2-popups';
import {PositionX,PositionY,ToastOpenArgs,ToastBeforeOpenArgs,ToastCloseArgs,ToastClickEventArgs} from "./toast";
import {ComponentModel} from '@syncfusion/ej2-base';

/**
 * Interface for a class ToastPosition
 */
export interface ToastPositionModel {

    /**
   * Specifies the position of the Toast notification with respect to the target container's left edge.



   */
    X?: PositionX | number | string;

    /**
   * Specifies the position of the Toast notification with respect to the target container's top edge.



   */
    Y?: PositionY | number | string;

}

/**
 * Interface for a class ButtonModelProps
 */
export interface ButtonModelPropsModel {

    /**
   * Specifies the Button component model properties to render the Toast action buttons.
   * ```html
   * <div id="element"> </div>
   * ```
   * ```typescript
   * let toast: Toast =  new Toast({ 
   *      buttons:
   *      [{ 
   *         model: { content:`Button1`, cssClass: `e-success` }
   *      }] 
   * });
   * toast.appendTo('#element');
   * ```
   *  

   */
    model?: ButtonModel;

    /**
   * Specifies the click event binding of action buttons created within Toast.
   * @event

   */
    click?: EmitType<Event>;

}

/**
 * Interface for a class ToastAnimations
 */
export interface ToastAnimationsModel {

    /**
   * Specifies the type of animation.


   */
    effect?: Effect;

    /**
   * Specifies the duration to animate.

   */
    duration?: number;

    /**
   * Specifies the animation timing function.

   */
    easing?: string;

}

/**
 * Interface for a class ToastAnimationSettings
 */
export interface ToastAnimationSettingsModel {

    /**
   * Specifies the animation to appear while showing the Toast.

   */
    show?: ToastAnimationsModel;

    /**
   * Specifies the animation to appear while hiding the Toast.

   */
    hide?: ToastAnimationsModel;

}

/**
 * Interface for a class Toast
 */
export interface ToastModel extends ComponentModel{

    /**
     * Specifies the width of the Toast in pixels/numbers/percentage. Number value is considered as pixels.
     * In mobile devices, default width is considered as `100%`. 


     */
    width?: string | number;

    /**
     * Specifies the height of the Toast in pixels/number/percentage. Number value is considered as pixels.


     */
    height?: string | number;

    /**
     * Specifies the title to be displayed on the Toast. 
     * Works only with string values.

     */
    title?: string;

    /**
     * Specifies the content to be displayed on the Toast. 
     * Accepts selectors, string values and HTML elements.


     */
    content?: string | HTMLElement;

    /**
     * Defines CSS classes to specify an icon for the Toast which is to be displayed at top left corner of the Toast.

     */
    icon?: string;

    /**
     * Defines single/multiple classes (separated by space) to be used for customization of Toast.

     */
    cssClass?: string;

    /**
     * Specifies the HTML element/element ID as a string that can be displayed as a Toast.
     * The given template is taken as preference to render the Toast, even if the built-in properties such as title and content are defined.

     */
    template?: string;

    /**
     * Specifies the newly created Toast message display order while multiple toast's are added to page one after another.
     * By default, newly added Toast will be added after old Toast's.

     */
    newestOnTop?: boolean;

    /**
     * Specifies whether to show the close button in Toast message to close the Toast.

     */
    showCloseButton?: boolean;

    /**
     * Specifies whether to show the progress bar to denote the Toast message display timeout.

     */
    showProgressBar?: boolean;

    /**
     * Specifies the Toast display time duration on the page in milliseconds. 
     * - Once the time expires, Toast message will be removed.
     * - Setting 0 as a time out value displays the Toast on the page until the user closes it manually.

     */
    timeOut?: number;

    /**
     * Specifies the Toast display time duration after interacting with the Toast. 

     */
    extendedTimeout?: number;

    /**
     * Specifies the animation configuration settings for showing and hiding the Toast.

     * hide: { effect: 'FadeOut', duration: 600, easing: 'linear' }}
     */
    animation?: ToastAnimationSettingsModel;

    /**
     * Specifies the position of the Toast message to be displayed within target container.
     * In the case of multiple Toast display, new Toast position will not update on dynamic change of property values
     * until the old Toast messages removed.
     * X values are: Left , Right ,Center
     * Y values are: Top , Bottom

     */
    position?: ToastPositionModel;

    /**
     * Specifies the collection of Toast action `buttons` to be rendered with the given
     * Button model properties and its click action handler.

     */
    buttons?: ButtonModelPropsModel[];

    /**
     * Specifies the target container where the Toast to be displayed.
     * Based on the target, the positions such as `Left`, `Top` will be applied to the Toast.
     * The default value is null, which refers the `document.body` element.



     */
    target?: HTMLElement | Element | string;

    /**
     * Triggers the event after the Toast gets created.
     * @event

     */
    created?: EmitType<Event>;

    /**
     * Triggers the event after the Toast gets destroyed.
     * @event

     */
    destroyed?: EmitType<Event>;

    /**
     * Triggers the event after the Toast shown on the target container.
     * @event

     */
    open?: EmitType<ToastOpenArgs>;

    /**
     * Triggers the event before the toast shown.
     * @event

     */
    beforeOpen?: EmitType<ToastBeforeOpenArgs>;

    /**
     * Trigger the event after the Toast hides.
     * @event

     */
    close?: EmitType<ToastCloseArgs>;

    /**
     * The event will be fired while clicking on the Toast.
     * @event

     */
    click?: EmitType<ToastClickEventArgs>;

}