import { ChildProperty } from '@syncfusion/ej2-base';
import { PointModel, DecoratorShapes } from '@syncfusion/ej2-drawings';
import { Size } from '@syncfusion/ej2-drawings';
import { PdfBoundsModel, PdfAnnotationBaseModel, PdfFontModel } from './pdf-annotation-model';
import { Container } from '@syncfusion/ej2-drawings';
import { PdfAnnotationType } from './enum';
import { ICommentsCollection, IReviewCollection } from '../pdfviewer';
/**
 * The `PdfBounds` is base for annotation bounds.
 * @hidden
 */
export declare abstract class PdfBounds extends ChildProperty<PdfBounds> {
    /**
     * Represents the the x value of the annotation.
     * @default 0
     */
    x: number;
    /**
     * Represents the the y value of the annotation.
     * @default 0
     */
    y: number;
    /**
     * Represents the the width value of the annotation.
     * @default 0
     */
    width: number;
    /**
     * Represents the the height value of the annotation.
     * @default 0
     */
    height: number;
    /**
     * Represents the the left value of the annotation.
     * @default 0
     */
    left: number;
    /**
     * Represents the the top value of the annotation.
     * @default 0
     */
    top: number;
    /**
     * Represents the the right value of the annotation.
     * @default 0
     */
    right: number;
    /**
     * Represents the the bottom value of the annotation.
     * @default 0
     */
    bottom: number;
    /**
     * Sets the reference point, that will act as the offset values(offsetX, offsetY) of a node
     * @default new Point(0,0)
     */
    location: PointModel;
    /**
     * Sets the size of the annotation
     * @default new Size(0, 0)
     */
    size: Size;
}
/**
 * The `PdfFont` is base for annotation Text styles.
 * @hidden
 */
export declare abstract class PdfFont extends ChildProperty<PdfFont> {
    /**
     * Represents the the font Bold style of annotation text content.
     * @default 'false'
     */
    isBold: boolean;
    /**
     * Represents the the font Italic style of annotation text content.
     * @default 'false'
     */
    isItalic: boolean;
    /**
     * Represents the the font Underline style of annotation text content.
     * @default 'false'
     */
    isUnderline: boolean;
    /**
     * Represents the the font Strikeout style of annotation text content.
     * @default 'false'
     */
    isStrikeout: boolean;
}
/**
 * Defines the common behavior of PdfAnnotationBase
 * @hidden
 */
export declare class PdfAnnotationBase extends ChildProperty<PdfAnnotationBase> {
    /**
     * Represents the unique id of annotation
     * @default ''
     */
    id: string;
    /**
     * Represents the annotation type of the pdf
     * @default 'Rectangle'
     */
    shapeAnnotationType: PdfAnnotationType;
    /**
     * Represents the measure type of the annotation
     * @default ''
     */
    measureType: string;
    /**
     * Represents the auther value of the annotation
     * @default ''
     */
    author: string;
    /**
     * Represents the modified date of the annotation
     * @default ''
     */
    modifiedDate: string;
    /**
     * Represents the subject of the annotation
     * @default ''
     */
    subject: string;
    /**
     * Represents the notes of the annotation
     * @default ''
     */
    notes: string;
    /**
     * Represents the stroke color of the annotation
     * @default 'black'
     */
    strokeColor: string;
    /**
     * Represents the fill color of the annotation
     * @default 'tranparent'
     */
    fillColor: string;
    /**
     * Represents the fill color of the annotation
     * @default 'tranparent'
     */
    stampFillColor: string;
    /**
     * Represents the stroke color of the annotation
     * @default 'black'
     */
    stampStrokeColor: string;
    /**
     * Represents the path data of the annotation
     * @default ''
     */
    data: string;
    /**
     * Represents the opecity value of the annotation
     * @default 1
     */
    opacity: number;
    /**
     * Represents the thickness value of annotation
     * @default 1
     */
    thickness: number;
    /**
     * Represents the border style of annotation
     * @default ''
     */
    borderStyle: string;
    /**
     * Represents the border dash array of annotation
     * @default ''
     */
    borderDashArray: string;
    /**
     * Represents the rotate angle of annotation
     * @default 0
     */
    rotateAngle: number;
    /**
     * Represents the annotation as cloud shape
     * @default false
     */
    isCloudShape: boolean;
    /**
     * Represents the cloud intensity
     * @default 0
     */
    cloudIntensity: number;
    /**
     * Represents the height of the leader of distance shapes
     * @default 40
     */
    leaderHeight: number;
    /**
     * Represents the line start shape style
     * @default null
     */
    lineHeadStart: string;
    /**
     * Represents the line end shape style
     * @default null
     */
    lineHeadEnd: string;
    /**
     * Represents vertex points in the line annotation or shape annotation.
     * @default []
     */
    vertexPoints: PointModel[];
    /**
     * Represents vertex points in the line annotation or shape annotation.
     * @default null
     */
    sourcePoint: PointModel;
    /**
     * Represents vertex points in the line annotation or shape annotation.
     * @default None
     */
    sourceDecoraterShapes: DecoratorShapes;
    /**
     * Represents vertex points in the line annotation or shape annotation.
     * @default None
     */
    taregetDecoraterShapes: DecoratorShapes;
    /**
     * Represents vertex points in the line annotation or shape annotation.
     * @default null
     */
    targetPoint: PointModel;
    /**
     * Represents vertex points in the line annotation or shape annotation.
     * @default []
     */
    segments: PointModel[];
    /**
     * Represents bounds of the annotation
     * @default new Point(0,0)
     */
    bounds: PdfBoundsModel;
    /**
     * Represents the cloud intensity
     * @default 0
     */
    pageIndex: number;
    /**
     * Represents the cloud intensity
     * @default -1
     */
    zIndex: number;
    /**
     * Represents the cloud intensity
     * @default null
     */
    wrapper: Container;
    /**
     * Represents the dynamic stamp
     * @default false
     */
    isDynamicStamp: boolean;
    /**
     * Represents the dynamic text.
     * @default ''
     */
    dynamicText: string;
    /**
     * Represents the unique annotName of the annotation
     * @default ''
     */
    annotName: string;
    /**
     * Represents the review collection of the annotation
     * @default ''
     */
    review: IReviewCollection;
    /**
     * Represents the comments collection of the annotation
     * @default []
     */
    comments: ICommentsCollection[];
    /**
     * Represents the comments collection of the annotation
     * @default '#000'
     */
    fontColor: string;
    /**
     * Represents the font size of the annotation content
     * @default '16'
     */
    fontSize: number;
    /**
     * Represents the font family of the annotation content
     * @default 'Helvetica'
     */
    fontFamily: string;
    /**
     * Represents the shape annotation label add flag
     * @default 'false'
     */
    enableShapeLabel: boolean;
    /**
     * Represents the shape annotation label content
     * @default 'label'
     */
    labelContent: string;
    /**
     * Represents the shape annotation label content fill color
     * @default '#ffffff00'
     */
    labelFillColor: string;
    /**
     * Represents the shape annotation label content max-length
     * @default '15'
     */
    labelMaxLength: number;
    /**
     * Represents the opecity value of the annotation
     * @default 1
     */
    labelOpacity: number;
    /**
     * Represents the shape annotation label content border color
     * @default '#ffffff00'
     */
    labelBorderColor: string;
    /**
     * Represents the text anlignment style of annotation
     * @default 'left'
     */
    textAlign: string;
    /**
     * Represents the text style of annotation
     * @default ''
     */
    font: PdfFontModel;
    /**
     * Represents the shape annotation label content bounds
     * @default ''
     */
    labelBounds: PdfBoundsModel;
    constructor(parent: any, propName: string, defaultValue: Object, isArray?: boolean);
}
/**
 * @hidden
 */
export declare class ZOrderPageTable {
    private pageIdTemp;
    /** @private */
    /** @private */
    pageId: number;
    private zIndexTemp;
    /** @private */
    /** @private */
    zIndex: number;
    private childNodesTemp;
    /** @private */
    /** @private */
    objects: PdfAnnotationBaseModel[];
    constructor();
}
