import { PdfViewer, PdfViewerBase } from '../index';
/**
 * The `ThumbnailView` module is used to handle thumbnail view navigation of PDF viewer.
 */
export declare class ThumbnailView {
    private pdfViewer;
    private pdfViewerBase;
    private previousElement;
    private thumbnailSelectionRing;
    private thumbnailImage;
    private isThumbnailCompleted;
    private startIndex;
    private thumbnailLimit;
    private thumbnailThreshold;
    private thumbnailTopMargin;
    private thumbnailRequestHandler;
    /**
     * @private
     */
    isThumbnailClicked: boolean;
    /**
     * @private
     */
    thumbnailView: HTMLElement;
    /**
     * @private
     */
    constructor(pdfViewer: PdfViewer, pdfViewerBase: PdfViewerBase);
    /**
     * @private
     */
    createThumbnailContainer(): void;
    /**
     * Open the thumbnail pane of the PdfViewer.
     * @returns void
     */
    openThumbnailPane(): void;
    /**
     * @private
     */
    createRequestForThumbnails(): Promise<any>;
    private requestCreation;
    /**
     * @private
     */
    gotoThumbnailImage(pageNumber: number): void;
    private checkThumbnailScroll;
    private getPageNumberFromID;
    private setFocusStyle;
    private renderThumbnailImage;
    private wireUpEvents;
    private unwireUpEvents;
    /**
     * @private
     */
    thumbnailClick: (event: MouseEvent) => void;
    private goToThumbnailPage;
    private setSelectionStyle;
    /**
     * @private
     */
    thumbnailMouseOver: (event: MouseEvent) => void;
    private setMouseOverStyle;
    /**
     * @private
     */
    thumbnailMouseLeave: (event: MouseEvent) => void;
    private setMouseLeaveStyle;
    private setMouseFocusStyle;
    private setMouseFocusToFirstPage;
    /**
     * @private
     */
    clear(): void;
    private getVisibleThumbs;
    private getVisibleElements;
    private binarySearchFirstItem;
    private backtrackBeforeAllVisibleElements;
    private getThumbnailElement;
    /**
     * @private
     */
    destroy(): void;
    /**
     * @private
     */
    getModuleName(): string;
}
/**
 * The `IVisibleThumbnailElement` module is used to handle visible thumbnail element collection of PDF viewer.
 * @hidden
 */
export interface IVisibleThumbnailElement {
    id: string;
    x: number;
    y: number;
    view: HTMLElement;
    percent: number;
}
/**
 * The `IVisibleThumbnail` module is used to handle visible thumbnail collection of PDF viewer.
 * @hidden
 */
export interface IVisibleThumbnail {
    first: IVisibleThumbnailElement;
    last: IVisibleThumbnailElement;
    views: Array<IVisibleThumbnailElement>;
}
