# Essential JS 2 - Modern JavaScript UI Components

The only JavaScript UI package you will ever need.

[![Screens of showcase applications](https://ej2.syncfusion.com/products/ej2-banner.gif)](https://syncfusion.com/products/javascript/)

*Screens of [showcase applications](#showcase-applications) created using Essential JS 2*

## Table of Contents

* [Framework highlights](#framework-highlights)
* [Control list](#control-list)
* [Resources](#resources)
* [Supported Frameworks](#supported-frameworks)
* [Showcase Applications](#showcase-applications)
* [Release Notes](#release-notes)
* [License](#license)

## Framework highlights

### Lightweight and user friendly

The entire Essential JS 2 framework is built from scratch to be lightweight and modular. Its footprint can be reduced further by including only the specific components and features your application requires.

### Modular architecture

All components have been built as modules to enable selective referencing, so only the components and features you need are included in your application.

### Built for performance

Performance is critical for delivering a good user experience. We ensure that all our components are designed and built to achieve the best performance possible.

### Responsive and touch friendly

All Essential JS 2 controls are touch friendly and render adaptively based on the device they are on to provide optimal usage experience on phones, tablets and desktops.

### Stunning built-in themes

Pixel-perfect built-in themes are available in material, bootstrap and fabric design. In addition, it comes with Accessible high-contrast theme and an online tool "[Theme Studio](https://ej2.syncfusion.com/themestudio/)" to customize the provided built-in themes.

### Globalization simplified

Easily build applications to be used by a global audience in various language and culture settings.

### Stay current

With our commitment to at least four major updates per year, you receive the most up-to-date functionality and new components in addition to monthly service packs and bug fixes. Custom patches are available as needed.


## Control List

* Grids
    * [**Data Grid**](https://syncfusion.com/products/javascript/grid?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/grid/grid-overview.html)
    * [**Pivot Grid**](https://www.syncfusion.com/products/javascript/pivot-table?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) (Preview) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/pivot-view/default.html)

* Editors
    * [**Rich Text Editor**](https://www.syncfusion.com/products/javascript/wysiwyg-rich-text-editor?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) (Preview) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/rich-text-editor/tools.html)
    * [**Document Editor**](https://www.syncfusion.com/products/javascript/word-processor?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) (Preview) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/document-editor/default.html)

* Dropdowns
    * [Autocomplete](https://www.syncfusion.com/products/javascript/autocomplete?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/auto-complete/default.html)
    * [Combobox](https://www.syncfusion.com/products/javascript/combobox?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/combo-box/default.html)
    * [**Dropdown List**](https://www.syncfusion.com/products/javascript/dropdown-list?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/drop-down-list/default.html)
    * [Multiselect Dropdown](https://www.syncfusion.com/products/javascript/multiselect-dropdown?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/multi-select/default.html)

* Inputs
    * [Form Validator](https://www.syncfusion.com/products/javascript/form-validator?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/form-validator/default.html)
    * [Textbox](https://www.syncfusion.com/products/javascript/textbox?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/textbox/default.html)
    * [Input Mask](https://www.syncfusion.com/products/javascript/input-mask?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/maskedtextbox/default.html)
    * [Numeric Textbox](https://www.syncfusion.com/products/javascript/numeric-textbox?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/numerictextbox/default.html)
    * [Radio Button](https://www.syncfusion.com/products/javascript/radio-button?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/button/radio-button.html)
    * [Checkbox](https://www.syncfusion.com/products/javascript/checkbox?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/button/check-box.html)
    * [Color Picker](https://www.syncfusion.com/products/javascript/color-picker?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/color-picker/default.html)
    * [File Upload](https://www.syncfusion.com/products/javascript/file-upload?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/uploader/default.html)
    * [Slider](https://www.syncfusion.com/products/javascript/slider?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/slider/default.html)
    * [Switch](https://www.syncfusion.com/products/javascript/switch?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/button/switch.html)

* Data Visualization
    * [**Charts**](https://www.syncfusion.com/products/javascript/charts?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/chart/line.html)
    * [Circular Gauge](https://www.syncfusion.com/products/javascript/gauge-charts?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/circular-gauge/default.html)
    * [**Diagram**](https://www.syncfusion.com/products/javascript/diagram?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) (Preview) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/diagram/default-functionalities.html)
    * [HeatMap](https://www.syncfusion.com/products/javascript/heatmap?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/heatmap/default.html)
    * [Linear Gauge](https://www.syncfusion.com/products/javascript/gauge-charts?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/linear-gauge/default.html)
    * [Maps](https://www.syncfusion.com/products/javascript/maps?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/maps/default.html)
    * [Range Selector](https://www.syncfusion.com/products/javascript/range-selector?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/range-navigator/default.html)
    * [Smith Chart](https://www.syncfusion.com/products/javascript/smith-chart?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/smith-chart/default.html)
    * [Sparkline](https://www.syncfusion.com/products/javascript/sparkline?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/sparkline/default.html)
    * [TreeMap](https://www.syncfusion.com/products/javascript/treemap?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/treemap/default.html)

* Calendars
    * [**Scheduler**](https://www.syncfusion.com/products/javascript/scheduler?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/schedule/default.html)
    * [Calendar](https://www.syncfusion.com/products/javascript/calendar?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/calendar/default.html)
    * [DatePicker](https://www.syncfusion.com/products/javascript/datepicker?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/datepicker/default.html)
    * [Date-Range Picker](https://www.syncfusion.com/products/javascript/date-range-picker?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/daterangepicker/default.html)
    * [Date-Time Picker](https://www.syncfusion.com/products/javascript/datetime-picker?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/datetimepicker/default.html)
    * [TimePicker](https://www.syncfusion.com/products/javascript/time-picker?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/timepicker/default.html)

* Navigation
    * [Accordion](https://www.syncfusion.com/products/javascript/accordion?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/accordion/default.html)
    * [**Context Menu**](https://www.syncfusion.com/products/javascript/context-menu?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/context-menu/default.html)
    * [**Menu**](https://www.syncfusion.com/products/javascript/menu?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) (Preview) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/menu/default.html)
    * [Sidebar](https://www.syncfusion.com/products/javascript/sidebar?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/sidebar/default.html)
    * [Tabs](https://www.syncfusion.com/products/javascript/tabs?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/tab/default.html)
    * [Toolbar](https://www.syncfusion.com/products/javascript/toolbar?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/toolbar/default.html)
    * [**TreeView**](https://www.syncfusion.com/products/javascript/treeview?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/treeview/default.html)

* Buttons
    * [Button](https://www.syncfusion.com/products/javascript/button?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/button/default.html)
    * [Button Group](https://www.syncfusion.com/products/javascript/button-group?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/button/button-group.html)
    * [Dropdown Menu](https://www.syncfusion.com/products/javascript/dropdown-menu?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/button/drop-down-button.html)
    * [Progress Button](https://www.syncfusion.com/products/javascript/progress-button?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) (Preview) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/button/progress-button.html)
    * [Split Button](https://www.syncfusion.com/products/javascript/split-button?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/button/split-button.html)

* Layout
    * [Avatar](https://www.syncfusion.com/products/javascript/avatar?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/avatar/default.html)
    * [Card](https://www.syncfusion.com/products/javascript/card?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/card/basic.html)
    * [Dialog](https://www.syncfusion.com/products/javascript/modal-dialog?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/dialog/default.html)
    * [**ListView**](https://www.syncfusion.com/products/javascript/listview?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/listview/default.html)
    * [Tooltip](https://www.syncfusion.com/products/javascript/tooltip?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/tooltip/default.html)

* Notification
    * [Badge](https://www.syncfusion.com/products/javascript/badge?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/badge/default.html)
    * [Toast](https://www.syncfusion.com/products/javascript/toast?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm) - [*Live demo*](https://ej2.syncfusion.com/demos#/material/toast/default.html)

## Supported Frameworks

* [Angular](https://www.syncfusion.com/products/angular?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm)
* [React](https://www.syncfusion.com/products/react?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm)
* [Vue.js](https://www.syncfusion.com/products/vue?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm)
* [ASP.NET Core](https://www.syncfusion.com/products/aspnetcore?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm)
* [ASP.NET MVC](https://www.syncfusion.com/products/aspnetmvc?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm)

## Resources

* [Documentation](https://ej2.syncfusion.com/documentation/?lang=typescript)
* [Theme Studio](https://ej2.syncfusion.com/themestudio/)
* [Custom Resource Generator](https://crg.syncfusion.com/)
* [Contact Support](https://www.syncfusion.com/support/directtrac/incidents/newincident?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm)
* [Forums](https://www.syncfusion.com/forums/essential-js2?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm)
* [What's New](https://www.syncfusion.com/products/whatsnew/essential-js2?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm)
* [Road Map](https://www.syncfusion.com/products/roadmap/essential-js2?utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm)
* [E-Books](https://www.syncfusion.com/ebooks?tag=javascript&utm_medium=listing&utm_source=npm&utm_campaign=ej2-common-npm)

## Showcase Applications

* [Diagram Builder](https://ej2.syncfusion.com/showcase/angular/diagrambuilder/)
* [Story Estimator](https://aspdotnetcore.syncfusion.com/showcase/aspnetcore/story-estimator/)
* [Stock Chart](https://ej2.syncfusion.com/showcase/angular/stockchart)
* [IT Asset Management](https://ej2.syncfusion.com/showcase/vue/assetmanagement/)
* [Expense Tracker](https://ej2.syncfusion.com/showcase/typescript/expensetracker/)
* [Web Mail](https://ej2.syncfusion.com/showcase/typescript/webmail/)
* [Loan Calculator](https://ej2.syncfusion.com/showcase/typescript/loancalculator/)

## Release Notes

Please refer this [link](https://ej2.syncfusion.com/documentation/release-notes/?lang=typescript)

## License

This is a commercial product and requires a paid license for possession or use. Syncfusion’s licensed software, including this component, is subject to the terms and conditions of Syncfusion's EULA (https://www.syncfusion.com/eula/es/). To acquire a license, you can purchase one at https://www.syncfusion.com/sales/products or start a free 30-day trial here (https://www.syncfusion.com/account/manage-trials/start-trials).

A free community license (https://www.syncfusion.com/products/communitylicense) is also available for companies and individuals whose organizations have less than $1 million USD in annual gross revenue and five or fewer developers.

    © Copyright 2019 Syncfusion, Inc. All Rights Reserved. 
    The Syncfusion Essential Studio license and copyright applies to this distribution.
